@@ -7,7 +7,7 @@ Computes a bunch of estimates from an event study data set:
 
 Does this through class FEEstimator
 '''
-import logging
+import warnings
 from pathlib import Path
 import pyamg
 import numpy as np
@@ -94,7 +94,11 @@ class FEEstimator:
 
                     statsonly (bool, default=False): if True, return only basic statistics
 
+                    feonly (bool, default=False): if True, compute only fixed effects and not variances
+
                     Q (str, default='cov(alpha, psi)'): which Q matrix to consider. Options include 'cov(alpha, psi)' and 'cov(psi_t, psi_{t+1})'
+
+                    seed (int, default=None): NumPy RandomState seed
         '''
         # Start logger
         logger_init(self)
@@ -116,10 +120,12 @@ class FEEstimator:
             'he': False, # If True, compute heteroskedastic correction
             'out': 'res_fe.json', # Outputfile where results are saved
             'statsonly': False, # If True, return only basic statistics
-            'Q': 'cov(alpha, psi)' # Which Q matrix to consider. Options include 'cov(alpha, psi)' and 'cov(psi_t, psi_{t+1})'
+            'feonly': False, # If True, compute only fixed effects and not variances
+            'Q': 'cov(alpha, psi)', # Which Q matrix to consider. Options include 'cov(alpha, psi)' and 'cov(psi_t, psi_{t+1})'
             # 'con': False, # Computes the smallest eigen values, this is the filepath where these results are saved FIXME not used
             # 'logfile': '', # Log output to a logfile FIXME not used
             # 'check': False # If True, compute the non-approximated estimates as well FIXME not used
+            'seed': None # np.random.RandomState() seed
         }
 
         self.params = update_dict(default_params, params)
@@ -137,6 +143,9 @@ class FEEstimator:
         self.res['ndp'] = self.ndraw_pii
         self.res['ndt'] = self.ndraw_trace
 
+        # Create NumPy Generator instance
+        self.rng = np.random.default_rng(self.params['seed'])
+
         # self.logger.info('FEEstimator object initialized')
 
     def __getstate__(self):
@@ -203,14 +212,16 @@ class FEEstimator:
 
         else: # If running analysis
             self.__create_fe_solver() # Solve FE model
-            self.__compute_trace_approximation_ho() # Compute trace approximation
+            self.__get_fe_estimates() # Add fixed effect columns
+            if not self.params['feonly']: # If running full model
+                self.__compute_trace_approximation_ho() # Compute trace approximation
 
-            # If computing heteroskedastic correction
-            if self.compute_he:
-                self.__compute_leverages_Pii() # Solve heteroskedastic model
-                self.__compute_trace_approximation_he() # Compute trace approximation
+                # If computing heteroskedastic correction
+                if self.compute_he:
+                    self.__compute_leverages_Pii() # Solve heteroskedastic model
+                    self.__compute_trace_approximation_he() # Compute trace approximation
 
-            self.__collect_res() # Collect all results
+                self.__collect_res() # Collect all results
 
         end_time = time.time()
 
@@ -219,6 +230,8 @@ class FEEstimator:
 
         self.__save_res() # Save results to json
 
+        self.__drop_cols() # Drop irrelevant columns
+
         self.logger.info('------ DONE -------')
 
     def __prep_vars(self):
@@ -515,7 +528,7 @@ class FEEstimator:
 
         # for r in trange(self.ndraw_trace):
         #     # Generate -1 or 1 - in this case length nn
-        #     Z = 2 * np.random.binomial(1, 0.5, self.nn) - 1
+        #     Z = 2 * self.rng.binomial(1, 0.5, self.nn) - 1
 
         #     # Compute either side of the Trace
         #     R_psi, R_alpha = self.__solve(Z)
@@ -538,8 +551,8 @@ class FEEstimator:
 
         for r in trange(self.ndraw_trace):
             # Generate -1 or 1
-            Zpsi = 2 * np.random.binomial(1, 0.5, self.nf - 1) - 1
-            Zalpha = 2 * np.random.binomial(1, 0.5, self.nw) - 1
+            Zpsi = 2 * self.rng.binomial(1, 0.5, self.nf - 1) - 1
+            Zalpha = 2 * self.rng.binomial(1, 0.5, self.nw) - 1
 
             R1 = Jq @ Zpsi
             psi1, alpha1 = self.__mult_AAinv(Zpsi, Zalpha)
@@ -583,8 +596,8 @@ class FEEstimator:
 
     #     for r in trange(self.ndraw_trace):
     #         # Generate -1 or 1
-    #         Zpsi = 2 * np.random.binomial(1, 0.5, self.nf - 1) - 1
-    #         Zalpha = 2 * np.random.binomial(1, 0.5, self.nw) - 1
+    #         Zpsi = 2 * self.rng.binomial(1, 0.5, self.nf - 1) - 1
+    #         Zalpha = 2 * self.rng.binomial(1, 0.5, self.nw) - 1
 
     #         R1 = Jq * Zpsi
     #         psi1, alpha1 = self.__mult_AAinv(Zpsi, Zalpha)
@@ -620,8 +633,8 @@ class FEEstimator:
 
     #     for r in trange(self.ndraw_trace):
     #         # Generate -1 or 1
-    #         Zpsi = 2 * np.random.binomial(1, 0.5, self.nf - 1) - 1
-    #         Zalpha = 2 * np.random.binomial(1, 0.5, self.nw) - 1
+    #         Zpsi = 2 * self.rng.binomial(1, 0.5, self.nf - 1) - 1
+    #         Zalpha = 2 * self.rng.binomial(1, 0.5, self.nw) - 1
 
     #         R1 = self.Jq * Zpsi
     #         psi1, alpha1 = self.__mult_AAinv(Zpsi, Zalpha)
@@ -644,8 +657,8 @@ class FEEstimator:
 
     #     for r in trange(self.ndraw_trace):
     #         # Generate -1 or 1
-    #         Zpsi = 2 * np.random.binomial(1, 0.5, self.nf - 1) - 1
-    #         Zalpha = 2 * np.random.binomial(1, 0.5, self.nw) - 1
+    #         Zpsi = 2 * self.rng.binomial(1, 0.5, self.nf - 1) - 1
+    #         Zalpha = 2 * self.rng.binomial(1, 0.5, self.nw) - 1
 
     #         R1 = self.J1 * Zpsi
     #         psi1, _ = self.__mult_AAinv(Zpsi, Zalpha)
@@ -667,8 +680,8 @@ class FEEstimator:
 
         for r in trange(self.ndraw_trace):
             # Generate -1 or 1
-            Zpsi = 2 * np.random.binomial(1, 0.5, self.nf - 1) - 1
-            Zalpha = 2 * np.random.binomial(1, 0.5, self.nw) - 1
+            Zpsi = 2 * self.rng.binomial(1, 0.5, self.nf - 1) - 1
+            Zalpha = 2 * self.rng.binomial(1, 0.5, self.nw) - 1
 
             psi1, alpha1 = self.__mult_AAinv(Zpsi, Zalpha)
             R2_psi = Jq * psi1
@@ -698,7 +711,7 @@ class FEEstimator:
 
         for r in trange(self.ndraw_trace):
             # Generate -1 or 1 - in this case length nn
-            Z = 2 * np.random.binomial(1, 0.5, self.nn) - 1
+            Z = 2 * self.rng.binomial(1, 0.5, self.nn) - 1
 
             # Compute Trace
             R_psi, R_alpha = self.__solve(Z, Dp2=False)
@@ -721,7 +734,7 @@ class FEEstimator:
         self.res['tot_var'] = self.tot_var
         self.res['eps_var_ho'] = self.var_e
         self.res['eps_var_fe'] = np.var(self.E)
-        self.res['var_y'] = self.__weighted_var(self.Yq, self.Dp)
+        # self.res['var_y'] = self.__weighted_var(self.Yq, self.Dp)
 
         # FE results
         # Plug-in variance
@@ -752,7 +765,7 @@ class FEEstimator:
         if self.compute_he:
             self.res['eps_var_he'] = self.Sii.mean()
             self.res['min_lev'] = self.adata.query('m == 1').Pii.min()
-            self.res['max_lev'] = self.adata.query('m == 1').Pii.max()
+            self.res['max_lev'] = self.res['max_lev'] # self.adata.query('m == 1').Pii.max() # Already computed, this just reorders the dictionary
             self.res['tr_var_he'] = np.mean(self.tr_var_he_all)
             self.res['tr_cov_he'] = np.mean(self.tr_cov_he_all)
             self.res['tr_var_ho_sd'] = np.std(self.tr_var_ho_all)
@@ -783,20 +796,18 @@ class FEEstimator:
 
         self.logger.info('saved results to {}'.format(self.params['out']))
 
-    def get_fe_estimates(self):
+    def __get_fe_estimates(self):
         '''
-        Return estimated psi_hats linked to firm ids and alpha_hats linked to worker ids.
-
-        Returns:
-            psi_hat_dict (dict): dictionary linking firm ids to estimated firm fixed effects
-            alpha_hat_dict (dict): dictionary linking worker ids to estimated worker fixed effects
+        Add the estimated psi_hats and alpha_hats to the dataframe.
         '''
         j_vals = np.arange(self.nf) # np.arange(self.adata.j1.max()) + 1
         i_vals = np.arange(self.nw) # np.arange(self.adata.i.max()) + 1
         psi_hat_dict = dict(zip(j_vals, np.concatenate([self.psi_hat, np.array([0])]))) # Add 0 for normalized firm
         alpha_hat_dict = dict(zip(i_vals, self.alpha_hat))
 
-        return psi_hat_dict, alpha_hat_dict
+        # Attach columns
+        self.adata['psi_hat'] = self.adata['j'].map(psi_hat_dict)
+        self.adata['alpha_hat'] = self.adata['i'].map(alpha_hat_dict)
 
     def __solve(self, Y, Dp1=True, Dp2=True):
         '''
@@ -940,18 +951,24 @@ class FEEstimator:
                 Pii += pp / len(Pii_all)
 
         I = 1.0 * self.adata.eval('m == 1')
-        max_leverage = (I * Pii).max()
+        self.res['max_lev'] = (I * Pii).max()
+
+        if self.res['max_lev'] >= 1:
+            leverage_warning = 'Max P_ii is {} which is >= 1. This should not happen - increase your value of ndraw_pii until this warning is no longer raised (ndraw_pii is currently set to {}).'.format(self.res['max_lev'], self.params['ndraw_pii'])
+            warnings.warn(leverage_warning)
+            self.logger.info(leverage_warning)
 
         # Attach the computed Pii to the dataframe
         self.adata['Pii'] = Pii
         self.logger.info('[he] Leverage range {:2.4f} to {:2.4f}'.format(self.adata.query('m == 1').Pii.min(), self.adata.query('m == 1').Pii.max()))
+        # print('Observation with max leverage:', self.adata[self.adata['Pii'] == self.res['max_lev']])
 
         # Give stayers the variance estimate at the firm level
         self.adata['Sii'] = self.Y * self.E / (1 - Pii)
         S_j = pd.DataFrame(self.adata).query('m == 1').rename(columns={'Sii': 'Sii_j'}).groupby('j')['Sii_j'].agg('mean')
 
-        self.adata = pd.merge(self.adata, S_j, on='j')
-        self.adata['Sii'] = np.where(self.adata['m'] == 1, self.adata['Sii'], self.adata['Sii_j'])
+        Sii_j = pd.merge(self.adata['j'], S_j, on='j')['Sii_j']
+        self.adata['Sii'] = np.where(self.adata['m'] == 1, self.adata['Sii'], Sii_j)
         self.Sii = self.adata['Sii']
 
         self.logger.info('[he] variance of residuals in heteroskedastic case: {:2.4f}'.format(self.Sii.mean()))
@@ -970,9 +987,17 @@ class FEEstimator:
 
         # Compute the different draws
         for r in trange(ndraw_pii):
-            R2 = 2 * np.random.binomial(1, 0.5, self.nn) - 1
+            R2 = 2 * self.rng.binomial(1, 0.5, self.nn) - 1
             Pii += 1 / ndraw_pii * np.power(self.__proj(R2, Dp0='sqrt', Dp2='sqrt'), 2.0)
 
         self.logger.info('done with batch')
 
         return Pii
+
+    def __drop_cols(self):
+        '''
+        Drop irrelevant columns (['Jq', 'Wq', 'Jq_row', 'Wq_row', 'Jq_col', 'Wq_col']).
+        '''
+        for col in ['Jq', 'Wq', 'Jq_row', 'Wq_row', 'Jq_col', 'Wq_col', 'Pii', 'Sii']:
+            if col in self.adata.columns:
+                self.adata.drop(col, inplace=True)
