@@ -42,6 +42,8 @@ class TwoWayMonteCarlo:
                 csig (float, default=1): standard error of sorting/network effects
 
                 p_move (float, default=0.5): probability a worker moves firms in each period
+
+                seed (int, default=None): NumPy RandomState seed
     '''
 
     def __init__(self, sim_params={}):
@@ -83,8 +85,12 @@ class TwoWayMonteCarlo:
 
                     statsonly (bool, default=False): if True, return only basic statistics
 
+                    feonly (bool, default=False): if True, compute only fixed effects and not variances
+
                     Q (str, default='cov(alpha, psi)'): which Q matrix to consider. Options include 'cov(alpha, psi)' and 'cov(psi_t, psi_{t+1})'
 
+                    seed (int, default=None): NumPy RandomState seed
+
             cre_params (dict): dictionary of parameters for CRE estimation
 
                 Dictionary parameters:
@@ -126,6 +132,8 @@ class TwoWayMonteCarlo:
 
                     i_t_how (str, default='max'): if 'max', keep max paying job; if 'sum', sum over duplicate worker-firm-year observations, then take the highest paying worker-firm sum; if 'mean', average over duplicate worker-firm-year observations, then take the highest paying worker-firm average. Note that if multiple time and/or firm columns are included (as in event study format), then duplicates are cleaned in order of earlier time columns to later time columns, and earlier firm ids to later firm ids
 
+                    data_validity (bool, default=True): if True, run data validity checks; much faster if set to False
+
                     copy (bool, default=False): if False, avoid copy
 
         Returns:
@@ -215,8 +223,12 @@ class TwoWayMonteCarlo:
 
                     statsonly (bool, default=False): if True, return only basic statistics
 
+                    feonly (bool, default=False): if True, compute only fixed effects and not variances
+
                     Q (str, default='cov(alpha, psi)'): which Q matrix to consider. Options include 'cov(alpha, psi)' and 'cov(psi_t, psi_{t+1})'
 
+                    seed (int, default=None): NumPy RandomState seed
+
             cre_params (dict): dictionary of parameters for CRE estimation
 
                 Dictionary parameters:
@@ -258,6 +270,8 @@ class TwoWayMonteCarlo:
 
                     i_t_how (str, default='max'): if 'max', keep max paying job; if 'sum', sum over duplicate worker-firm-year observations, then take the highest paying worker-firm sum; if 'mean', average over duplicate worker-firm-year observations, then take the highest paying worker-firm average. Note that if multiple time and/or firm columns are included (as in event study format), then duplicates are cleaned in order of earlier time columns to later time columns, and earlier firm ids to later firm ids
 
+                    data_validity (bool, default=True): if True, run data validity checks; much faster if set to False
+
                     copy (bool, default=False): if False, avoid copy
         '''
         # Initialize NumPy arrays to store results
